(function($){

    var l27BfugJs = {
        vars: typeof l27BfugJsVars != 'undefined' ? l27BfugJsVars : {},
        formData: typeof l27BfugFormData != 'undefined' ? l27BfugFormData : {},
        init: function() {
            if ($('.booking-form-wrapper').length > 0) {
                this.processBookingForm();
            }
        },
        processBookingForm: function() {
            var _this = this;
            var btn = $('#btnSaveBookingForm');
            var wrapper = $('.booking-form-wrapper');
            var thisForm = wrapper.find('form');
            var ajax_placeholder = $('<div style="text-align: center;"><img src="' + _this.vars.plugin_dir_url + '/assets/img/ajax-loader.svg"><h3><em>Loading form data...</em></h3></div>');

            var lblService = wrapper.find('#lblService');
            var fldService = wrapper.find('#fldService');
            var lblFrequency = wrapper.find('#lblFrequency');
            var fldFrequency = wrapper.find('#fldFrequency');
            var lblDiscountCode = wrapper.find('#lblDiscountCode');
            var fldDiscountCode = wrapper.find('#fldDiscountCode');
            var fldTotalPrice = wrapper.find('[name="total_price"]');

            var wrpTotalPrice = wrapper.find('.total-cost-wrapper');
            var loaderTotalPrice = wrpTotalPrice.find('.overlay-loader-wrapper');

            var rowTotalPrice = wrpTotalPrice.find('#row-total');
            var lblTotalPrice = rowTotalPrice.find('#label');
            var valTotalPrice = rowTotalPrice.find('#value');

            var rowSubTotal = wrpTotalPrice.find('#row-sub-total');
            var lblSubTotal = rowSubTotal.find('#label');
            var valSubTotal = rowSubTotal.find('#value');

            var rowDiscountTotal = wrpTotalPrice.find('#row-discount');
            var lblDiscountTotal = rowDiscountTotal.find('#label');
            var valDiscountTotal = rowDiscountTotal.find('#value');

            var rowFutureTotal = wrpTotalPrice.find('#row-future-total');
            var lblFutureTotal = rowFutureTotal.find('#label');
            var valFutureTotal = rowFutureTotal.find('#value');

            var rowTodaysTotal = wrpTotalPrice.find('#row-todays-total');
            var lblTodaysTotal = rowTodaysTotal.find('#label');
            var valTodaysTotal = rowTodaysTotal.find('#value');

            $(document).ready(function(){
                wrpTotalPrice.scrollToFixed({marginTop: 50, minWidth: 400, dontSetWidth: true});
            });

            var services = [];
            var frequencies = [];
            var form = {};

            var required_ajax_requests = ['services', 'frequencies', 'form'];
            var completed_ajax_requests = [];

            beforeAjax();

            $.get(ajaxurl, {'action': _this.vars.ajax_action, 'path': 'booking/services'}, function(response){
                services = JSON.parse(response);
                completed_ajax_requests.push('services');
                afterAjax();
            });

            $.get(ajaxurl, {'action': _this.vars.ajax_action, 'path': 'booking/frequencies'}, function(response){
                frequencies = JSON.parse(response);
                completed_ajax_requests.push('frequencies');
                afterAjax();
            });

            $.get(ajaxurl, {'action': _this.vars.ajax_action, 'path': 'booking/form'}, function(response){
                form = JSON.parse(response);
                completed_ajax_requests.push('form');
                afterAjax();
            });

            btn.on('click', function(){
                thisForm.submit();
            });

            function beforeAjax() {
                wrapper.find('form, .total-cost-wrapper').hide();
                wrapper.after(ajax_placeholder);
                disableFormSubmit();
            }

            function afterAjax() {
                if (required_ajax_requests.length == completed_ajax_requests.length) {
                    ajax_placeholder.fadeOut('slow', function(){
                        wrapper.find('form, .total-cost-wrapper').fadeIn('slow');
                    });
                    populateBookingForm();
                }
            }

            function disableFormSubmit() {
                btn.attr('disabled', 'disabled');
            }

            function enableFormSubmit() {
                btn.removeAttr('disabled');
            }

            thisForm.on('change', 'input, select', function(e){
                calculateTotalPrice();
            });

            function populateBookingForm() {
                lblService.html('Service');
                fldService.append('<option></option>');
                $.each(services, function(key, value){
                    var strSelected = (_this.formData.hasOwnProperty('service_id') && _this.formData.service_id == value.id) ? ' selected="selected"' : '';
                    fldService.append('<option value="' + value.id + '" data-service-index="' + key + '" ' + strSelected + '>' + value.name + '</option>');
                    if (strSelected) {
                        populatePricingParams(key);
                        populateHourly(key);
                        populateExtras(key);
                    }
                });

                lblFrequency.html('Frequency');
                $.each(frequencies, function(key, value){
                    var strSelected = _this.formData.frequency_id == value.id ? ' selected="selected"' : '';
                    fldFrequency.append('<option value="' + value.id + '" ' + strSelected + '>' + value.name + '</option>');
                });

                lblDiscountCode.html('Discount Code');
                if (_this.formData.rcode) {
                    fldDiscountCode.val(_this.formData.rcode);
                }

                fldService.on('change', function(){
                    $(this).find('option:selected').each(function(){
                        var selectedServiceIndex = $(this).data('service-index');
                        populatePricingParams(selectedServiceIndex);
                        populateHourly(selectedServiceIndex);
                        populateExtras(selectedServiceIndex);
                    });
                });

                calculateTotalPrice();
            }

            function populatePricingParams(service_index) {
                var wrpPricingParams = wrapper.find('#wrpPricingParams');
                wrpPricingParams.html('');
                if (typeof service_index != 'undefined') {
                    var pricing_params = typeof services[service_index] != 'undefined' ? services[service_index]['pricing_parameters'] : [];
                    if (typeof pricing_params != 'undefined' && pricing_params.length > 0) {
                        $.each(pricing_params, function(key, value){
                            var el = $('<div class="form-group"></div>');
                            el.append('<label for="fldPricingParams' + key + '" id="lblPricingParam' + key + '">' + value.name + '</label>');
                            var select = $('<select class="text-field" name="pricing_parameter[' + value.id + ']"></select>');
                            for(var i = value.quantity_minimum; i <= value.quantity_maximum; i++) {
                                var strSelected = (_this.formData.hasOwnProperty('pricing_parameter') && _this.formData.pricing_parameter[value.id] == i) ? ' selected="selected"' : '';
                                select.append('<option value="' + i + '" ' + strSelected + '>' + i + '</option>');
                            }
                            el.append(select);
                            wrpPricingParams.append(el);
                        });
                    }
                }
            }

            function populateExtras(service_index) {
                var wrpExtras = wrapper.find('#wrpExtras');
                wrpExtras.html('');
                if (typeof service_index != 'undefined') {
                    var extras = typeof services[service_index] != 'undefined' ? services[service_index].extras : [];
                    if (typeof extras != 'undefined' && extras.length > 0) {
                        wrpExtras.append('<div class="form-group"><label>Extras</label></div>');
                        $.each(extras, function(key, value){
                            var el = $('<div class="form-group"></div>');
                            var strChecked = (_this.formData.hasOwnProperty('extra') && _this.formData.extra.hasOwnProperty(value.id) && _this.formData.extra[value.id]) ? ' checked="checked"' : '';
                            if (value.hasOwnProperty('quantity_based') && value.quantity_based) {
                                var val = (_this.formData.hasOwnProperty('extra') && _this.formData.extra.hasOwnProperty(value.id)) ? _this.formData.extra[value.id] : '';
                                el.append('<label><input type="checkbox" ' + strChecked + ' class="reset-my-sibling-on-check-off"> <input class="check-my-sibling-on-change" type="number" name="extra[' + value.id + ']" value="' + val + '" style="width: 70px;"> ' + value.name + '</label>');
                            }
                            else {
                                el.append('<label><input type="checkbox" name="extra[' + value.id + ']" value="1" ' + strChecked + '> ' + value.name + '</label>');
                            }
                            wrpExtras.append(el);
                        });
                    }
                }

                $('.reset-my-sibling-on-check-off').on('change', function(e){
                    var el = $(e.currentTarget);
                    var sibling = el.siblings('input');
                    var checked = el.is(':checked');
                    if (!checked) {
                        sibling.val('');
                    }
                    else {
                        sibling.val('1');
                    }
                });

                $('.check-my-sibling-on-change').on('change', function(e){
                    var el = $(e.currentTarget);
                    var sibling = el.siblings('input[type="checkbox"]');
                    sibling.prop('checked', true);
                });
            }

            function populateHourly(service_index) {
                var wrpHourly = wrapper.find('#wrpHourly');
                wrpHourly.html('');
                if (typeof service_index != 'undefined') {
                    var hourly = typeof services[service_index] != 'undefined' ? services[service_index].hourly : [];
                    if (typeof hourly != 'undefined') {
                        var el1 = $('<div class="form-group"></div>');
                        var select = $('<select class="text-field" name="hourly[quantity]"></select>');
                        for(var i = hourly.quantity_minimum; i <= hourly.quantity_maximum; i++) {
                            var strSelected = (_this.formData.hasOwnProperty('hourly') && _this.formData.hourly['quantity'] == i) ? ' selected="selected"' : '';
                            select.append('<option value="' + i + '" ' + strSelected + '>' + i + ' ' + form.system_fields.service_workers.label + (i != 1 ? 's' : '') + '</option>');
                        }
                        el1.append(select);

                        var el2 = $('<div class="form-group"></div>');
                        var select = $('<select class="text-field" name="hourly[minutes]"></select>');
                        var hh_ratio = (services[service_index].hourly.half_hour_increments) ? 0.5 : 1;
                        for(var i = hourly.minutes_minimum; i <= hourly.minutes_maximum; i += (hourly.minutes_minimum*hh_ratio)) {
                            var strSelected = (_this.formData.hasOwnProperty('hourly') && _this.formData.hourly['minutes'] == i) ? ' selected="selected"' : '';
                            select.append('<option value="' + i + '" ' + strSelected + '>' + (i/hourly.minutes_minimum) + ' Hour' + ((i/hourly.minutes_minimum) != 1 ? 's' : '') + '</option>');
                        }

                        el2.append(select);
                        var elWrapper = $('<div class="inline-form-group"></div>');
                        elWrapper.append(el1);
                        elWrapper.append(el2);
                        wrpHourly.append(elWrapper);
                    }
                }
            }

            function calculateTotalPrice() {
                if (services && frequencies && fldService.val() && fldFrequency.val()) {
                    enableFormSubmit();

                    var frmData = thisForm.serializeArray();
                    var selections = $.grep(frmData, function(v) {
                        return ['autopath', 'path', 'id', 'total_price'].indexOf(v.name) == -1;
                    });

                    var selected = {};
                    $.each(selections, function(key, selection){
                        var name = selection.name;
                        if (name.indexOf('[') != -1) {
                            var nbits = name.split('[');
                            var name = nbits[0];
                            var index = nbits[1].replace(']', '');
                            if (!selected.hasOwnProperty(name)) {
                                selected[name] = [];
                            }
                            selected[name][index] = selection.value;
                        }
                        else {
                            selected[name] = selection.value;
                        }
                    });

                    var total_price = 0;
                    var service_index, frequency_index;
                    var price_estimate_request_data = {services: []};
                    var price_estimate_services = {};

                    /*============== Core Pricing ==============*/

                    var search = searchAssocArray(services, 'id', selected.service_id);
                    if (search && search.hasOwnProperty('result') && search.result.hasOwnProperty('price')) {
                        total_price += search.result.price;
                        service_index = search.index;
                        price_estimate_services.id = search.result.id;
                        price_estimate_services.pricing_parameters = [];
                        price_estimate_services.extras = [];
                    }

                    if (selected.hasOwnProperty('pricing_parameter') && selected.pricing_parameter.length > 0) {
                        $.each(selected.pricing_parameter, function(k, v){
                            if (v) {
                                var search = searchAssocArray(services[service_index]['pricing_parameters'], 'id', k);
                                if (search) {
                                    total_price += (search.result.price*v);
                                    price_estimate_services.pricing_parameters.push({id: search.result.id, quantity: parseInt(v)});
                                }
                            }
                        });
                    }

                    if (selected.hasOwnProperty('extra') && selected.extra.length > 0) {
                        $.each(selected.extra, function(k, v){
                            if (v) {
                                var search = searchAssocArray(services[service_index]['extras'], 'id', k);
                                if (search) {
                                    total_price += (search.result.price*v);
                                    price_estimate_services.extras.push({id: search.result.id, quantity: parseInt(v)});
                                }
                            }
                        });
                    }

                    if (selected.hasOwnProperty('hourly')) {
                        if (selected.hourly.hasOwnProperty('minutes') && selected.hourly.minutes && selected.hourly.hasOwnProperty('quantity') && selected.hourly.quantity) {
                            price_estimate_services.hourly = {quantity: parseInt(selected.hourly.quantity), minutes: parseInt(selected.hourly.minutes)};
                        }
                    }

                    /*============== Frequency-Based Discount ==============*/

                    var search = searchAssocArray(frequencies, 'id', selected.frequency_id);
                    if (search && search.hasOwnProperty('result') && search.result.hasOwnProperty('percent')) {
                        var discount = total_price*(search.result.percent/100);
                        total_price -= discount;
                        frequency_index = search.index;
                        price_estimate_request_data.frequency_id = search.result.id;
                    }

                    /*=============== API-Based Pricing ================*/
                    if (fldDiscountCode.val()) {
                        price_estimate_request_data.discount_code = fldDiscountCode.val();
                    }

                    price_estimate_request_data.service_date = new Date().toISOString().split('.')[0];
                    price_estimate_request_data.services.push(price_estimate_services);

                    var ajax_args = {
                        'path': 'booking/estimate_price',
                        'json_data': JSON.stringify(price_estimate_request_data),
                        'method': 'POST',
                        'is_json': true
                    };

                    loaderTotalPrice.fadeIn();
                    disableFormSubmit();

                    $.post(ajaxurl + '?action=' + _this.vars.ajax_action, ajax_args, function(response){
                        var price_estimates = JSON.parse(response);
                        total_price = price_estimates.total;

                        if (price_estimates.hasOwnProperty('discount_amount') && price_estimates.discount_amount) {
                            valDiscountTotal.html(formatAmount(price_estimates.discount_amount));
                            valSubTotal.html(formatAmount(price_estimates.extras + price_estimates.services));
                            rowDiscountTotal.show();
                            rowSubTotal.show();
                        }
                        else {
                            rowDiscountTotal.hide();
                            rowSubTotal.hide();
                        }

                        if (price_estimates.hasOwnProperty('next_total') && price_estimates.next_total) {
                            valFutureTotal.html(formatAmount(price_estimates.next_total));
                            rowFutureTotal.show();

                            valTodaysTotal.html(formatAmount(price_estimates.total));
                            rowTodaysTotal.show();

                            rowTotalPrice.hide();
                        }
                        else {
                            rowFutureTotal.hide();
                            rowTodaysTotal.hide();
                            rowTotalPrice.show();
                        }

                        valTotalPrice.html(formatAmount(total_price));
                        fldTotalPrice.val(total_price);

                        enableFormSubmit();
                        loaderTotalPrice.fadeOut();
                    });
                }
                else {
                    wrpTotalPrice.children().each(function(){
                        if ($(this).find('#value').length > 0) {
                            $(this).find('#value').html(formatAmount(0));
                            if ($(this).attr('id') != 'row-total') {
                                $(this).hide();
                            }
                            else {
                                $(this).show();
                            }
                        }
                    });

                    if (fldDiscountCode.val()) {
                        enableFormSubmit();
                    }
                    else {
                        disableFormSubmit();
                    }
                }
            }

            function formatAmount(amount) {
                return amount.toFixed(2).replace(/(\d)(?=(\d{3})+\.)/g, '$1,');
            }

            function searchAssocArray(array, key, value) {
                var out = {};
                for(var i = 0; i < array.length; i++) {
                    if (array[i][key] == value) {
                        out = {index: i, result: array[i]};
                        break;
                    }
                }
                return out;
            }
        },
    }

    l27BfugJs.init();

})(jQuery);