<?php

namespace l27Bfug;

class Requests extends Base {
    const host = 'launch27.com';
    const api_version = 1;
    private static $default_request_params = array(
        'method' => 'GET',
        'is_json' => true,
    );

    private static $subdomain;

    public function __construct() {

    }

    public function set_subdomain($subdomain = NULL) {
        if (!$subdomain) {
            $subdomain = isset(self::$settings['subdomain']) && self::$settings['subdomain'] ? self::$settings['subdomain'] : self::$subdomain;
        }
        self::$subdomain = $subdomain;
    }

    public function authenticate_user($email, $password) {
        $json_data = json_encode(array('email' => $email, 'password' => $password));
        $args = array(
            'method' => 'POST',
            'json_data' => $json_data,
        );
        $request = self::make_request('login', $args);

        if ($request && $request['info']['http_code'] == 200) {
            return $request['response'];
        }
    }

    public function get_user_settings() {
        $request = self::make_request('settings');
        if ($request && $request['info']['http_code'] == 200) {
            return $request['response'];
        }
    }

    public function get_booking_form() {
        $request = self::make_request('booking/form', array('method' => 'GET'));
        if ($request && $request['info']['http_code'] == 200) {
            return $request['response'];
        }
    }

    public static function make_request($path = NULL, $params = array()) {
        if ($path) {
            self::set_subdomain();
            $url = 'https://' . self::$subdomain . '.' . self::host . '/v'. self::api_version . '/' . $path;
        }
        else {
            if (isset($params['url']) && $params['url']) {
                $url = $params['url'];
            }
        }

        if (!$url) {
            return false;
        }

        $params = array_merge(self::$default_request_params, $params);
        $urlparts = parse_url($url);
        if (empty($urlparts)) return false;
        @$ssl = ($urlparts['scheme'] == "https") ? true : false;
        $method = (isset($params['method'])) ? $params['method'] : "GET";
        $method = strtoupper($method);
        $is_json = false;
        if (isset($params['data']) && !empty($params['data'])) {
            if (is_array($params['data'])) {
                if (isset($params['is_json']) && $params['is_json'] == true) {
                    $strdata = json_encode($params['data']);
                    $is_json = true;
                }
                else {
                    $strdata = http_build_query($params['data'],'','&');
                }
            }
            else {
                $strdata = $params['data'];
            }
            if ($method == "GET") $url .= "?".$strdata;
        }
        if (isset($params['json_data']) && $params['json_data']) {
            $strdata = stripslashes($params['json_data']);
            $is_json = true;
        }

        $headers = array();

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$url);
        if (version_compare(PHP_VERSION, '5.4', '<')) {
            curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
        }
        if (isset($params['basic_authentication']) && $params['basic_authentication']) {
            curl_setopt($ch, CURLOPT_USERPWD,$params['basic_authentication']);
        }
        if ($is_json) {
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            $headers[] = 'Content-Type: application/json; charset=utf-8';
        }
        curl_setopt($ch, CURLOPT_HEADER, false);
        // curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        if (strtoupper($method) == "POST") {
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $strdata);
        }
        if (isset($params['cookie']) && isset($params['cookiefile'])) {
            $cookiefile = $params['cookiefile'];
            curl_setopt($ch, CURLOPT_COOKIEJAR, $cookiefile);
            curl_setopt($ch, CURLOPT_COOKIEFILE, $cookiefile);
        }
        curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows NT 6.1; rv:23.0) Gecko/20100101 Firefox/23.0");
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        if ($ssl) {
            $sslverifyhost = 2;
            if (isset($params['ignore_ssl_verify']) && $params['ignore_ssl_verify']) $sslverifyhost = 0;
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, $sslverifyhost);
        }

        $result['response'] = curl_exec($ch);
        $result['info'] = curl_getinfo($ch);
        $result['error'] = curl_error($ch);
        curl_close($ch);
        return $result;
    }
}