<?php

namespace l27Bfug;

class RoutesController extends Base {

    public function login() {
        if ($_POST) {
            $validate_settings = array(
                'subdomain' => array(
                    'name' => 'Subdomain',
                    'rules' => array('required')
                ),
                'email' => array(
                    'name' => 'Email',
                    'rules' => array('required', 'email')
                ),
                'password' => array(
                    'name' => 'Password',
                    'rules' => array('required')
                )
            );

            $validate = l27_bfug_validate_form_field($_POST, $validate_settings);
            if ($validate['status'] != 'success') {
                l27_bfug_set_flash_message('error', $validate['response']);
            }
            else {
                // $_POST = l27_bfug_sanitize_data($_POST);
                $subdomain = str_replace('.' . \l27Bfug\Requests::host, '', $_POST['subdomain']);
                \l27Bfug\Requests::set_subdomain($subdomain);
                $auth = \l27Bfug\Requests::authenticate_user($_POST['email'], $_POST['password']);
                if ($auth) {
                    $user_settings = \l27Bfug\Requests::get_user_settings();
                    l27_bfug_update_settings(array_merge(json_decode($user_settings, true), array('auth' => $auth, 'subdomain' => $subdomain)));
                    l27_bfug_set_flash_message('success', 'Login successful');
                    l27_bfug_redirect_to_route();
                }
                else {
                    l27_bfug_set_flash_message('error', 'Login unsuccessful');
                }
            }
        }
    }

    public function logout() {
        global $wpdb;
        $sql = "TRUNCATE " . L27_BFUG_SETTINGS_TABLE;
        $sql = $wpdb->prepare($sql);
        $wpdb->query($sql);
        l27_bfug_redirect_to_route();
    }

    public function settings() {
        if ($_POST) {
            $validate_settings = array(
                'booking_page_url' => array(
                    'name' => 'Booking page URL',
                    'rules' => array('required', 'url')
                )
            );

            $validate = l27_bfug_validate_form_field($_POST, $validate_settings);
            if ($validate['status'] != 'success') {
                l27_bfug_set_flash_message('error', $validate['response']);
            }
            else {
                l27_bfug_update_settings($_POST);
                l27_bfug_set_flash_message('success', 'Settings saved');
                l27_bfug_redirect_to_route('settings');
            }
        }
    }

    public function forms_list() {
        global $wpdb;

        $page = isset($_GET['l24_bfug_page']) ? absint($_GET['l24_bfug_page']) : 1;

        $limit = self::listings_per_page;
        $offset = ($page-1)*$limit;
        $total = $wpdb->get_var("SELECT COUNT(id) FROM " . L27_BFUG_FORMS_TABLE);
        $number_of_pages = ceil($total/$limit);

        $rows = $wpdb->get_results( "SELECT * FROM " . L27_BFUG_FORMS_TABLE . " ORDER BY id DESC LIMIT $offset, $limit");

        $links = paginate_links( array(
            'base' => add_query_arg( 'l24_bfug_page', '%#%' ),
            'format' => '',
            'prev_text' => __( '&laquo;', 'text-domain' ),
            'next_text' => __( '&raquo;', 'text-domain' ),
            'total' => $number_of_pages,
            'current' => $page
        ) );

        $out = array('rows' => $rows, 'pagination' => array('page' => $page, 'number_of_pages' => $number_of_pages, 'links' => $links));

        return $out;
    }

    public function forms_add() {
        global $wpdb;

        if ($_POST) {
            $_SESSION[L27_BFUG_SLUG]['form_data'] = $_POST;
            if (!$_POST['path']) {
                $_POST['path'] = $_POST['autopath'] = self::generate_auto_path();
            }

            if ($_POST['path'] != $_POST['autopath']) {
                if (!preg_match('/^[A-Za-z0-9._-]+$/', $_POST['path'])) {
                    l27_bfug_set_flash_message('error', 'Form URL has invalid characters.');
                    l27_bfug_redirect_to_route('forms_add');
                }
                $available = l27_bfug_check_path_available($_POST['path']);
                if (!$available) {
                    l27_bfug_set_flash_message('error', 'Form URL is unavailable. Kindly choose a different one or leave it blank.');
                    l27_bfug_redirect_to_route('forms_add');
                }
            }

            $params = $_POST;
            if (isset($params['path'])) unset($params['path']);
            if (isset($params['autopath'])) unset($params['autopath']);
            if (isset($params['total_price'])) unset($params['total_price']);
            if (isset($params['id'])) unset($params['id']);

            if (!array_filter($params)) {
                l27_bfug_set_flash_message('error', 'Blank form submitted');
                l27_bfug_redirect_to_route('forms_add');
            }

            $sql = "INSERT INTO " . L27_BFUG_FORMS_TABLE . " (path, params, total_price) VALUES (%s, %s, %f)";
            $sql = $wpdb->prepare($sql, $_POST['path'], json_encode($params), $_POST['total_price']);
            $wpdb->query($sql);

            if (isset($_SESSION[L27_BFUG_SLUG]['form_data'])) unset($_SESSION[L27_BFUG_SLUG]['form_data']);

            l27_bfug_set_flash_message('success', 'Booking form saved');
            l27_bfug_redirect_to_route();
        }
        else {
            if ($_GET) {
                $path = self::generate_auto_path();
                $form_data = isset($_SESSION[L27_BFUG_SLUG]['form_data']) ? $_SESSION[L27_BFUG_SLUG]['form_data'] : array();
                if ($form_data) wp_localize_script('l27-bfug-scripts', 'l27BfugFormData', $form_data);
                return array('path' => $path, 'form_data' => $form_data);
            }
        }
    }

    public function forms_edit() {
        global $wpdb;

        self::$current_view = 'forms_add';

        if (!isset($_REQUEST['id'])) {
            l27_bfug_set_flash_message('error', 'No booking form selected');
            l27_bfug_redirect_to_route();
        }

        $row = $wpdb->get_row("SELECT params, path, total_price FROM " . L27_BFUG_FORMS_TABLE . " WHERE id='{$_REQUEST['id']}'");
        if (!$row) {
            l27_bfug_set_flash_message('error', 'Booking form not found');
            l27_bfug_redirect_to_route();
        }

        $form_params = json_decode($row->params, true);
        $form_params['id'] = $_REQUEST['id'];
        $form_params['path'] = $row->path;
        $form_params['total_price'] = $row->total_price;

        if ($_POST) {
            $_SESSION[L27_BFUG_SLUG]['form_data'] = $_POST;
            if (!$_POST['path']) {
                $_POST['path'] = $_POST['autopath'] = self::generate_auto_path();
            }

            if ($_POST['path'] != $_POST['autopath']) {
                if (!preg_match('/^[A-Za-z0-9._-]+$/', $_POST['path'])) {
                    l27_bfug_set_flash_message('error', 'Form URL has invalid characters.');
                    l27_bfug_redirect_to_route('forms_add');
                }

                $available = l27_bfug_check_path_available($_POST['path']);
                if (!$available) {
                    l27_bfug_set_flash_message('error', 'Form URL is unavailable. Kindly choose a different one or leave it blank.');
                    l27_bfug_redirect_to_route('forms_edit', array('id' => $_REQUEST['id']));
                }
            }

            $params = $_POST;
            if (isset($params['path'])) unset($params['path']);
            if (isset($params['autopath'])) unset($params['autopath']);
            if (isset($params['total_price'])) unset($params['total_price']);
            if (isset($params['id'])) unset($params['id']);

            if (!array_filter($params)) {
                l27_bfug_set_flash_message('error', 'Blank form submitted');
                l27_bfug_redirect_to_route('forms_edit');
            }

            $sql = "UPDATE " . L27_BFUG_FORMS_TABLE . " SET path='%s', params='%s', total_price='%f' WHERE id='{$_REQUEST['id']}'";
            $sql = $wpdb->prepare($sql, $_POST['path'], json_encode($params), $_POST['total_price']);
            $wpdb->query($sql);

            if (isset($_SESSION[L27_BFUG_SLUG]['form_data'])) unset($_SESSION[L27_BFUG_SLUG]['form_data']);

            l27_bfug_set_flash_message('success', 'Booking form updated');
            l27_bfug_redirect_to_route();
        }
        else {
            if ($_GET) {
                $form_data = (isset($_SESSION[L27_BFUG_SLUG]['form_data']) && $_SESSION[L27_BFUG_SLUG]['form_data']['id'] == $form_params['id']) ? $_SESSION[L27_BFUG_SLUG]['form_data'] : $form_params;
                if ($form_data) wp_localize_script('l27-bfug-scripts', 'l27BfugFormData', $form_data);
                return array('path' => $row->path, 'total_price' => $row->total_price, 'form_data' => $form_data, 'id' => $_REQUEST['id']);
            }
        }
    }

    private function generate_auto_path() {
        global $wpdb;

        $path = NULL;

        $next_record_id_row = $wpdb->get_row("SELECT auto_increment FROM INFORMATION_SCHEMA.TABLES WHERE table_name = '" . L27_BFUG_FORMS_TABLE . "'");
        $next_record_id = $next_record_id_row->auto_increment;

        while(!$path) {
            $tmp_path = str_replace('=', '', base64_encode($next_record_id));
            $available = l27_bfug_check_path_available($tmp_path);
            if ($available) {
                $path = $tmp_path;
            }
            $next_record_id++;
        }

        return $path;
    }

    public function forms_delete() {
        global $wpdb;

        if (isset($_GET['id'])) {
            $sql = "DELETE FROM " . L27_BFUG_FORMS_TABLE . " WHERE id = %d";
            $sql = $wpdb->prepare($sql, $_GET['id']);
            $wpdb->query($sql);
            l27_bfug_set_flash_message('success', 'Booking form deleted successfully');
        }
        l27_bfug_redirect_to_route();
    }
}