<?php

function l27_bfug_routes_handler() {
    $page = @$_GET['page'];
    $route = NULL;

    if (strpos($page, L27_BFUG_SLUG . '_') !== false) {
        $route = str_replace(L27_BFUG_SLUG . '_', '', $page);
    }

    if (!$route) {
        $route = \l27Bfug\Base::default_route;
    }

    if (method_exists('\l27Bfug\RoutesController', $route)) {
        \l27Bfug\Base::$current_route = $route;
        \l27Bfug\Base::hook_before();
        $rc = new \l27Bfug\RoutesController();
        $vars = $rc->$route();
        l27_bfug_render_controller_view($vars);
    }
}

function l27_bfug_api_handler() {

    \l27Bfug\Requests::init();
    $api_params = array(
        'data' => @$_REQUEST['params'],
        'json_data' => @$_REQUEST['json_data'],
        'method' => (isset($_REQUEST['method']) ? $_REQUEST['method'] : 'GET'),
        'is_json' => (isset($_REQUEST['is_json']) ? $_REQUEST['is_json'] : false),
    );

    $api_request = \l27Bfug\Requests::make_request($_REQUEST['path'], $api_params);
    if ($api_request) {
        print $api_request['response'];
    }
}

function l27_bfug_render_controller_view($vars = array()) {
    $view = (\l27Bfug\Base::$current_view) ? \l27Bfug\Base::$current_view : \l27Bfug\Base::$current_route;

    $view_file = L27_BFUG_DIR . '/views/' . str_replace('_', '-', $view) . '.html';

    if (file_exists($view_file)) {
        $views[] = 'header';
        if (isset(\l27Bfug\Base::$settings['auth'])) {
            $views[] = 'left-sidebar';
        }
        $views[] = $view;
        $views[] = 'footer';
        l27_bfug_render_view($views, $vars);
    }
}

function l27_bfug_render_view($views, $vars = array()) {
    if (!is_array($views)) {
        $views = array($views);
    }

    $views = array_filter($views);

    if ($views) {
        if (is_array($vars) && $vars) {
            extract($vars);
        }

        foreach($views as $view) {
            $view_file = L27_BFUG_DIR . '/views/' . str_replace('_', '-', $view) . '.html';
            if (file_exists($view_file)) {
                include($view_file);
            }
        }

        $js_vars['plugin_url'] = l27_bfug_get_route_url();
        $js_vars['plugin_dir_url'] = L27_BFUG_URL;
        $js_vars['api_url'] = l27_bfug_get_route_url('api_request');
        $js_vars['ajax_action'] = L27_BFUG_AJAX_ACTION;
        wp_localize_script('l27-bfug-scripts', 'l27BfugJsVars', $js_vars);
    }
}

function l27_bfug_get_settings() {
    global $wpdb;

    $out = array();
    $settings = $wpdb->get_results("SELECT * FROM " . L27_BFUG_SETTINGS_TABLE);
    if ($settings) {
        foreach($settings as $setting) {
            if (l27_bfug_is_json($setting->settings_value)) {
                $setting->settings_value = json_decode($setting->settings_value, true);
            }
            $out[$setting->settings_key] = $setting->settings_value;
        }
    }
    return $out;
}

function l27_bfug_update_settings($data) {
    global $wpdb;

    if ($data && is_array($data)) {
        foreach($data as $key => $value) {
            if (is_array($value)) {
                $value = json_encode($value);
            }

            $sql = "INSERT INTO " . L27_BFUG_SETTINGS_TABLE . " (settings_key, settings_value) VALUES (%s,%s) ON DUPLICATE KEY UPDATE settings_value = %s";
            $sql = $wpdb->prepare($sql, $key, $value, $value);
            $wpdb->query($sql);
        }
    }
}

function l27_bfug_redirect_to_route($route = null, $args = array()) {
    $url = l27_bfug_get_route_url($route, $args);
    header('Location: ' . $url);
    exit;
}

function l27_bfug_get_route_url($route = null, $args = array()) {
    $vars['page'] = L27_BFUG_SLUG;

    if ($route) {
        $vars['page'] .= '_' . $route;
    }

    if ($args) {
        $vars = array_merge($vars, $args);
    }

    $url = '?' . http_build_query($vars);

    return $url;
}

function l27_bfug_set_flash_message($type, $msg) {
    $_SESSION[L27_BFUG_SLUG]['messages'][$type] = $msg;
}

function l27_bfug_get_flash_message() {
    if (isset($_SESSION[L27_BFUG_SLUG]['messages']) && $_SESSION[L27_BFUG_SLUG]['messages']) {
        $msgs = array();
        foreach($_SESSION[L27_BFUG_SLUG]['messages'] as $type => $msg) {
            $msgs[$type] = $msg;
        }
        l27_bfug_render_view('flash_messages', array('flash_messages' => $msgs));
        $_SESSION[L27_BFUG_SLUG]['messages'] = array();
    }
}

function l27_bfug_validate_form_field($data, $settings) {
    $errors = array();
    $out = array();
    if ($settings) {
        foreach($settings as $field => $field_settings) {
            $rules = $field_settings['rules'];
            $name = $field_settings['name'];

            if (in_array('required', $rules)) {
                if (!isset($data[$field]) || !$data[$field]) {
                    $errors[] = $name . ' field is required';
                }
            }

            if (in_array('email', $rules)) {
                if (isset($data[$field]) && $data[$field] && !l27_bfug_validate_email($data[$field])) {
                    $errors[] = $name . ' field is not a valid email format';
                }
            }

            if (in_array('url', $rules)) {
                if (isset($data[$field]) && $data[$field] && !l27_bfug_validate_url($data[$field])) {
                    $errors[] = $name . ' field is not a valid URL format';
                }
            }
        }
    }

    if ($errors) {
        $out['status'] = 'error';
        $out['response'] = $errors;
    }
    else {
        $out['status'] = 'success';
    }

    return $out;
}

function l27_bfug_validate_email($email) {
    return !!filter_var($email, FILTER_VALIDATE_EMAIL);
}

function l27_bfug_validate_url($url) {
    return !filter_var($url, FILTER_VALIDATE_URL) === false;
}

function l27_bfug_sanitize_data($data) {
    $out = $data;
    if ($data && is_array($data)) {
        foreach($data as $k => $v) {
            $out[$k] = l27_bfug_sanitize_string($v);
        }
    }
    return $out;
}

function l27_bfug_sanitize_string($string) {
    $out = $string;
    $out = stripslashes($out);
    $out = htmlspecialchars($out);
    $out = strip_tags($out);
    return $out;
}

function l27_bfug_is_json($str) {
    $out = json_decode($str) != null;
    return $out;
}

function l27_bfug_handle_short_url_request($path) {
    global $wpdb;

    if ($path) {
        $path_data = l27_bfug_check_path($path);
        if ($path_data) {
            new \l27Bfug\Base();

            $params = json_decode($path_data->params, true);
            $booking_page_url = \l27Bfug\Base::$settings['booking_page_url'];

            if (!$booking_page_url) {
                $booking_page_url = 'https://' . \l27Bfug\Base::$settings['subdomain'] . '.' . \l27Bfug\Requests::host;
            }

            if ($booking_page_url) {
                $sql = "UPDATE " . L27_BFUG_FORMS_TABLE . " SET clicks=clicks+1 WHERE id='%d'";
                $sql = $wpdb->prepare($sql, $path_data->id);
                $wpdb->query($sql);

                $query = http_build_query($params);
                $redir_url = $booking_page_url . '?' . $query;
                header('Location: ' . $redir_url);
                exit;
            }
        }
    }
}

function l27_bfug_check_path($path) {
    global $wpdb;

    if ($path) {
        $row = $wpdb->get_row("SELECT id, params FROM " . L27_BFUG_FORMS_TABLE . " WHERE path='$path'");
        if ($row) {
            return $row;
        }
    }
}

function l27_bfug_check_path_available($path) {
    global $wpdb;

    if ($path) {
        $path_exists = l27_bfug_check_path($path);
        if ($path_exists) {
            return false;
        }
        $url = home_url($path);
        $req = \l27Bfug\Requests::make_request(null, array('url' => $url));
        if ($req) {
            $exists = false;
            if (substr($req['info']['http_code'], 0, 1) == '2') {
                $exists = true;
            }
            elseif (substr($req['info']['http_code'], 0, 1) == '3') {
                if (strpos($req['info']['redirect_url'], $req['info']['url']) !== false) {
                    $exists = true;
                }
            }
            if ($exists) {
                return false;
            }
        }
        return true;
    }
}

?>