<?php

add_action('init', 'l27_bfug_plugin_init');

function l27_bfug_plugin_init() {
    if (!session_id()) {
        session_start();
    }
    ob_start();
}

add_action( 'template_redirect', 'l27_bfug_intercept_redirection' );

function l27_bfug_intercept_redirection(){
    global $wp_query;
    if ($wp_query->is_404){
        $path = @$wp_query->query['name'];
        if (!$path) $path = @$wp_query->query['pagename'];
        if (!$path) $path = $wp_query->query_vars['name'];
        if (!$path) {
            $full_path = @$_SERVER['REQUEST_URI'];
            $path_bits = explode('/', $full_path);
            $path = end($path_bits);
        }
        l27_bfug_handle_short_url_request($path);
    }
}

register_activation_hook(L27_BFUG_PLUGIN_FILE, 'l27_bfug_plugin_install');

function l27_bfug_plugin_install() {
    global $wpdb;

    if (version_compare(PHP_VERSION, '5.3', '<' )) {
        wp_die('<p>The <strong>' . L27_BFUG_NAME . '</strong> plugin requires PHP version 5.3 or greater. Your PHP version is ' . PHP_VERSION . ', which is not currently supported.</p>', 'Plugin Activation Error',  array('response' => 200, 'back_link' => TRUE ));
    }

    if (!is_callable('curl_init')) {
        wp_die('<p>The <strong>' . L27_BFUG_NAME . '</strong> plugin requires PHP\'s cURL Library . You currently do not have this installed or it has been disabled.</p>', 'Plugin Activation Error',  array('response' => 200, 'back_link' => TRUE ));
    }

    $charset_collate = $wpdb->get_charset_collate();

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

    $sql = "CREATE TABLE IF NOT EXISTS " . L27_BFUG_SETTINGS_TABLE . " (
        id smallint(5) unsigned NOT NULL AUTO_INCREMENT,
        settings_key varchar(100) NOT NULL,
        settings_value varchar(2000) NOT NULL,
        PRIMARY KEY id (id),
        UNIQUE KEY settings_key (settings_key)
    ) $charset_collate;";

    dbDelta( $sql );

    $sql = "CREATE TABLE IF NOT EXISTS " . L27_BFUG_FORMS_TABLE . " (
        id int(10) unsigned NOT NULL AUTO_INCREMENT,
        path varchar(10) NOT NULL,
        params varchar(600) NOT NULL,
        clicks int(10) unsigned NOT NULL,
        total_price decimal(12,2) unsigned NOT NULL,
        PRIMARY KEY id (id),
        UNIQUE KEY path (path)
    ) $charset_collate;";

    dbDelta( $sql );

    $wpdb->query('ALTER TABLE '. L27_BFUG_FORMS_TABLE.' CHANGE `path` `path` VARCHAR(100)');
}

register_uninstall_hook(L27_BFUG_PLUGIN_FILE, 'l27_bfug_plugin_uninstall');

function l27_bfug_plugin_uninstall() {
    global $wpdb;
    $wpdb->query("DROP TABLE IF EXISTS " . L27_BFUG_SETTINGS_TABLE);
    $wpdb->query("DROP TABLE IF EXISTS " . L27_BFUG_FORMS_TABLE);
}

add_action("admin_enqueue_scripts", "l27_bfug_register_scripts", 11);

function l27_bfug_register_scripts() {
    wp_enqueue_script('l27-bfug-scripts', L27_BFUG_URL . 'assets/js/scripts.js', array( 'jquery' ), null, true );
    wp_enqueue_script('l27-bfug-scrolltofixed', L27_BFUG_URL . 'assets/js/jquery-scrolltofixed-min.js', array( 'jquery' ), null, true );
    wp_enqueue_style('l27-bfug-styles', L27_BFUG_URL . 'assets/css/styles.css', false, '1.0.0');
}

add_action( 'wp_ajax_l27_bfug_ajax_request', 'l27_bfug_ajax_request' );

function l27_bfug_ajax_request() {
    l27_bfug_api_handler();
    exit;
}

?>